HTCPhAsOutput <- function(fit, phiCoefName="^Ph", TiCoefName="^Ti", TeCoefName="^Te", flipPhCoefSign=TRUE){
  ## Calculate, using Langrange weighting, the minimum variance estimate of a single steady-state
  ## gain from two estimated transfer functions for which the steady-state gain represents the same.
  ##
  ## This can for example be for a thermal model of a building with thermostatic heating. Using the
  ## model with heat flow rate as output, and both internal and external temperature as input, then
  ## two transfer functions can be estimated, which represents the same steady-state gain, namely the
  ## heat loss coefficient of the building.

##  browser()
  ## Find the indexes for the coefficients for the inputs and output
  iPh <- grep(phiCoefName, names(fit$coefficients))
  iOmegaTi <- grep(TiCoefName, names(fit$coefficients))
  iOmegaTe <- grep(TeCoefName, names(fit$coefficients))
  ## Number of coefficients
  np <- length(fit$coefficients)

  ## Flip the sign of the phi coefficients if specified (In an ARX model estimated with lm their sign is flipped)
  if(flipPhCoefSign){ fit$coefficients[iPh] <- -fit$coefficients[iPh] }

  ## The AR (phi) and input (omega) polynomials with a 1 through
  phiSum <- sum(c(1,fit$coefficients[iPh]))
  omegaTiSum <- sum(fit$coefficients[iOmegaTi])
  omegaTeSum <- sum(fit$coefficients[iOmegaTe])

  ## The estimated steady-state gain for each transfer function
  Hi <- omegaTiSum / phiSum
  He <- -omegaTeSum / phiSum

  ## The Jacobian, note that it contains an element for each coefficient, hence many zeroes
  J <- matrix(0,nrow=2,ncol=np)
  J[1, iPh] <- -omegaTiSum / phiSum^2
  J[1, iOmegaTi] <- 1 / phiSum
  J[1, iOmegaTe] <- 0
  J[2, iPh] <- omegaTeSum / phiSum^2
  J[2, iOmegaTi] <- 0
  J[2, iOmegaTe] <- -1 / phiSum
  ##print(J)

  ## The covariance matrix of the estimated parameters
  ## If from the fit is a list (from ARMAX)
  if(class(fit)=="list") covar <- fit$covar
  else covar <- vcov(fit)
  ## Flip the AR sign is specified
  if(flipPhCoefSign)
    {
      covar[iPh,] <- covar[iPh,] * -1
      covar[,iPh] <- covar[,iPh] * -1
    }
  
  ## The covariance matrix of the H values
  VarHs <- J %*% covar %*% t(J)
  ## Calculate the weighting coefficient which gives the minimum variance linear weighting
  lambda <- (VarHs[2,2] - VarHs[1,2]) / (VarHs[1,1] + VarHs[2,2] - 2*VarHs[1,2])

  ## The H estimate
  H <- lambda * Hi + (1-lambda) * He
  sdH <- sqrt( (VarHs[1,1]* VarHs[2,2]-VarHs[1,2]^2)/(VarHs[1,1]+VarHs[2,2]-2*VarHs[1,2]) )

  ## ## Calculate the g value similarly
  ## g <- B.3/B.2

  ## jac.g.s <- matrix(0,nrow=1,ncol=np)
  ## jac.g.s[1,iG] <- B.2
  ## jac.g.s[1,iQi] <- -B.3

  ## jac.g <- jac.g.s/B.2^2
  ## Varg <- jac.g%*%covar%*%t(jac.g)
  ## sd.g <- sqrt(Varg)  

  ## ## The covariance between Uc and g
  ## jac.Usg <- rbind(jac.us,jac.g)
  ## Var.Usg <- jac.Usg%*%covar%*%t(jac.Usg)
  ## ## Use: cov(a*X+b*Y,Z) = a*cov(X,Y) + b*cov(Y,Z)
  ## cov.Ug <- lambda * Var.Usg[1,3] + (1-lambda) * Var.Usg[2,3]
  ## ## The correlation
  ## cor.Ug <- cov.Ug/(sd.U*sd.g)
  
  return(list(Hi = Hi, He = He, H = H, sdH = sdH, VarHs = VarHs))
}
