analyzeGBFit <- function(fit, tPer=NA, plotACF=FALSE, plotSeries=FALSE, plotMore=FALSE, newdev=FALSE, acfRmNA=FALSE, acftext="", printSummary=FALSE,...){
  ## Take out the model,fit, and data from the list modelAndFit
  model <- fit$model
  Dat   <- fit$data[[1]]

  ## See the summary of the estimation result
  if(printSummary) print(summary(fit,extended=TRUE))

  ##----------------------------------------------------------------
  ## Calculate the one-step predictions of the state (i.e. the residuals)
  tmp <- predict(fit)[[1]]

  ## Calculate the residuals and put them with the data in a data.frame X
  Dat$yTiHat <- tmp$output$pred$yTi
  Dat$residuals <- Dat$yTi - Dat$yTiHat
  Dat$TiSd <- tmp$state$sd$Ti
  Dat$yTiSd <- tmp$output$sd$yTi
  Dat$residualsStd <- Dat$residuals/Dat$yTiSd
  
  ## ## The timedate for plotting
  ## Dat$timedate <- Dat$timedate

  ## Cut out a period if specified
  if(is.na(tPer[1])) 
    X <- Dat
  else
    X <- Dat[per(tPer[1],Dat$timedate,tPer[2]),]
  ##----------------------------------------------------------------
  
  if(plotSeries)
    {
      ##----------------------------------------------------------------
      ## Time series plots of the inputs and residuals
      ## A new plotting window?
      if(newdev) dev.new()
      ## Plot the time series (see "functions/plotTSBeg.R" to see the plot setup function)
      setpar("ts", mfrow=c(4,1))
      gridSeq <- seq(asP("2009-01-01"),by="days",len=365)
      ##
      plot(X$timedate,X$Ph,type="n",xlab="",ylab="Inputs",yaxt="n",ylim=c(0,1))
      abline(v=gridSeq,h=0,col="grey85",lty=3)
      lines(X$timedate,norm(X$Ph),...)
      lines(X$timedate,norm(X$Te),col=2,...)
      lines(X$timedate,norm(X$Isol),col=3,...)
      ##                lines(X$timedate,norm(X$Tadj),col=4,...)
      legend("bottomright",c("$\\Phi_{\\mathrm{h}}$","$T_{\\mathrm{e}}$","$I_{\\mathrm{sol}}$"),lty=1,col=1:3,cex=0.7,bg="grey95")
      ##                legend("topright",c(paste(range(X$Ph),collapse=" to "),paste(range(X$Te),collapse=" to "),paste(range(X$Tadj),collapse=" to "),paste(range(X$G),collapse=" to ")),lty=1,col=1:4,cex=0.7,bg="grey95")
      ##
      plot(X$timedate,X$yTi,ylim=range(X[,c("yTi","yTiHat")],na.rm=TRUE),type="n",xlab="",ylab="$T_{\\mathrm{i}} ~(^{\\circ}$C)",yaxt="n")
      axis(2,pretty(scalerange(X[,c("yTi","yTiHat")],0.2)))
      abline(v=gridSeq,h=0,col="grey85",lty=3)
      lines(X$timedate,X$yTi,...)
      lines(X$timedate,X$yTiHat,col=2,...)
      legend("topright",c("Measured","Predicted"),lty=1,col=1:2,bg="grey95",cex=0.8)
      ## 
      plot(X$timedate,X$residuals,ylab="$\\hat{\\varepsilon}$ ~($^\\circ$C)",type="n",xlab="",yaxt="n")
      axis(2,pretty(scalerange(X$residuals,0.2)))
      abline(v=gridSeq,h=0,col="grey92")
      lines(X$timedate,X$residuals,...)
      lines(X$timedate,X$yTiSd,col=4,...)
      lines(X$timedate,-X$yTiSd,col=4,...)
      legend("topright",c("$\\hat{\\sigma}_k$"),lty=1,col=1,bg="grey95",cex=0.8)
      ## Plot the standardized residuals
      plot(X$timedate,X$residuals/X$yTiSd,ylab="$\\hat{\\varepsilon}/\\hat{\\sigma}$",type="n",xlab="",yaxt="n")
      axis(2,pretty(scalerange(X$residuals/X$yTiSd,0.2)))
      abline(v=gridSeq,h=0,col="grey92")
      lines(X$timedate,X$residuals/X$yTiSd,...)
      mtext(paste("log-likelihood =",sprintf("%.1f",fit$loglik)),line=-2)
      ##
      axis.POSIXct(1,X$timedate,format="%Y-%m-%d",xaxt="s")
    }

    
  if(plotACF)
    {
      res <- X$residuals/X$yTiSd
      ##----------------------------------------------------------------
      ## Plot the auto-correlation function and cumulated periodogram
      ## Open a new plot window?
      if(newdev) dev.new()
      ##
      par(mfrow=c(1,2),mar=c(3.2,3.2,3,1),mgp=c(2.2,0.7,0),cex.lab=0.7)
      ## The blue lines indicates the 95% confidence interval, meaning that if it is
      ##  white noise, then approximately 19 out of 20 lag correlations will be inside.
      if(acfRmNA) res <- res[!is.na(res)]
      ##
      Ts <- as.numeric(diff(Dat$timedate[3:4]), units="hours")
      acf(res, lag.max=24*1/Ts+10, main="", xlab="")
      title(main="ACF of std. residuals",line=0.5)
      title(main=acftext,line=-2,cex.main=2)
      ## The periodogram is the estimated energy spectrum in the signal
      ##spec.pgram(res, main="Raw periodogram")
      ## The cumulated periodogram
      par(mgp=c(20,0.7,0))
      cpgram(res,main="")
      title(main="Cumulated periodogram",line=0.5)
      ##----------------------------------------------------------------
    }

  ##----------------------------------------------------------------
  ## Calculate the loglikelihood value
  if(printSummary) print(paste("Loglikelihood", format(fit$loglik,digits=2)))
  ##----------------------------------------------------------------

  ##----------------------------------------------------------------
  ## The estimated HTC-value for the TiThTe model
  i <- which(names(fit$xm)%in%fit$Rnames)
  HTC <- 1/sum(fit$xm[i])
  if(printSummary) print(paste("HTC:",format(HTC,digits=2))) ## W/C
  ## The covariance for the two estimated R values
  ii <- which(colnames(fit$corr)%in%fit$Rnames)
  if(length(i)==1) cov <- fit$sd[i] * fit$corr[ii,ii] * fit$sd[i]
  else cov <- diag(fit$sd[i]) %*% fit$corr[ii,ii] %*% diag(fit$sd[i])
  ##
  ## if(printSummary) print(paste("Cov = ",cov))

  ## Calculate the uncertainty of the HTC value with a linear approximation to the covariance
  ## The Jacobian, the derived of the HTC-value with respect to each estimate in fit$xm[i]
  J <- t( sapply(1:length(i), function(ii,x){ -1/sum(x)^2 }, x=fit$xm[i]) )
  ## The estimated variance of HTC
  varHTC <- J %*% cov %*% t(J)
  ## and standard deviance
  sdHTC <- sqrt(varHTC)
  ## Return the confidence interval
  if(printSummary) print(paste("HTC 95% confidence band:",paste(format(c(HTC-1.96*sdHTC,HTC+1.96*sdHTC),digits=2),collapse=" to ")))

  ## The gA value
  gA <- fit$xm["gA"]
  names(gA) <- NULL
  sdgA <- fit$sd["gA"]
  names(sdgA) <- NULL
  if(printSummary) print(paste("gA:",format(gA,digits=2))) ## W/C
  if(printSummary) print(paste("gA 95% confidence band:",paste(format(c(gA-1.96*sdgA, gA+1.96*sdgA),digits=2),collapse=" to ")))

  ## ## Calculate the uncertainty of the HTC value with a simulation approach
  ## ## Needed for multivariate normal distribution simulation
  ## require(MASS)
  ## ## Generate multivariate normal random values
  ## Rsim <- mvrnorm(n=1000000,mu=fit$xm[i],Sigma=cov)
  ## ## For each realization calculate the HTC-value
  ## HTCsim <- 1/apply(Rsim,1,sum)
  ## ## Estimate the 2.5% and 97.5% quantiles of the simulated values as a confidence interval
  ## quantile(HTCsim,probs=c(0.025,0.975))
  ## ##----------------------------------------------------------------

  if(plotMore)
    {
      ##----------------------------------------------------------------
      ## Time series plots residuals and more statistics
      ## A new plotting window?
      if(newdev) dev.new()
      ## Plot the time series (see "functions/plotTSBeg.R" to see the plot setup function)
      plotTSBeg(4)
      gridSeq <- seq(asP("2009-01-01"),by="days",len=365)
      ## Plot the standardized residuals
      plot(X$timedate,X$residuals/X$yTiSd,ylab="Standardized residuals",type="n",xlab="",yaxt="n")
      axis(2,pretty(scalerange(X$residuals/X$yTiSd,0.2)))
      abline(v=gridSeq,h=0,col="grey92")
      lines(X$timedate,X$residuals/X$yTiSd,...)
      ##

                                        # c er hubers psi
      psiLim <- model$options$hubersPsiLimit
                                        #                ressq <- X$residuals^2 / X$yTiSd^2
                                        #                ressq[which(ressq>psiLim^2)]<-psiLim*(2*sqrt(ressq[which(ressq>psiLim^2)])-psiLim)
                                        #(nll<- -( 0.5 * sum(log(X$residuals^2)+ressq,na.rm=TRUE) + 0.5 * sum(!is.na(X$residuals))*log(2*pi) ))

      ## ressq<-X$residuals^2/X$yTiSd^2
      ## ressq[which(ressq>psiLim^2)]<-psiLim*(2*sqrt(ressq[which(ressq>psiLim^2)])-psiLim)
      ## plot(ressq)
      ## (nll<--sum(log(X$yTiSd^2)+ressq))
      ##browser()
      print(fit$loglik)
      
      ressq <- X$residuals^2/X$yTiSd^2
      ressq[which(ressq>psiLim^2)]<-psiLim*(2*sqrt(ressq[which(ressq>psiLim^2)])-psiLim)
      print(nll <-  0.5*sum(log(X$yTiSd^2)+ressq,na.rm=TRUE) + 0.5*sum(!is.na(X$residuals))*log(2*pi) )

      ## The 
      X$loglik <- 0.5*(log(X$yTiSd^2)+ressq)+0.5*log(2*pi)*!is.na(X$residuals)
      plot(X$timedate,X$loglik,ylab="likelihood",type="n",xlab="",yaxt="n")
      axis(2,pretty(scalerange(X$loglik,0.2)))
      abline(v=gridSeq,h=0,col="grey92")
      lines(X$timedate,X$loglik,...)
      ##
      plot(X$timedate,X$residuals,ylab="Residuals ($^{\\circ}$C)",type="n",xlab="",yaxt="n")
      axis(2,pretty(scalerange(X$residuals,0.2)))
      abline(v=gridSeq,h=0,col="grey92")
      lines(X$timedate,X$residuals,...)
      lines(X$timedate,X$yTiSd,col=4,...)
      lines(X$timedate,-X$yTiSd,col=4,...)
      lines(X$timedate,X$TiSd,col=2,...)
      lines(X$timedate,-X$TiSd,col=2,...)
      ##
      plot(X$timedate,X$yTi,ylim=range(X[,c("yTi","yTiHat")],na.rm=TRUE),type="n",xlab="",ylab="yTi, yTiHat ($^{\\circ}$C)",yaxt="n")
      axis(2,pretty(scalerange(X[,c("yTi","yTiHat")],0.2)))
      abline(v=gridSeq,h=0,col="grey85",lty=3)
      lines(X$timedate,X$yTi,...)
      lines(X$timedate,X$yTiHat,col=2,...)
      legend("bottomright",c("Measured","Predicted"),lty=1,col=1:2,bg="grey95")
    }

  ##
  R <- list()
  R$HTC <- HTC
  R$sdHTC <- sdHTC
  R$residualsStd <- Dat$residualsStd
  R$gA <- gA
  R$sdgA <- sdgA
  
  invisible(R)
}

