## @knitr greyboxForIDEE_prepareData
## Prepare the data for the example and write it to a file
## This part is not included in the document
source("sourceFunctions.R")

## Function for loading the data
readIDEEdata <- function(name, t0, plotit=FALSE){
  X <- read.table(paste0("dataIDEE/",name,".dat"), header=TRUE, sep=";", na.strings="NaN")
  X$t <- X$t * 3600
  X <- X[X$t>0,]
  X$t <- asP(t0) + X$t
  ## Naming
  names(X) <- gsub("_",".",names(X))
  names(X)[names(X) == "Ta"] <- "Te"
  names(X)[names(X) == "yTi"] <- "Ti"
  names(X)[names(X) == "G.hor.h"] <- "G"
  ## Preprocess
  X$Te.lim[X$Te.lim < -20] <- NA
  ##
  i <- grep("^T|^yT",names(X)); X[ ,i][X[ ,i]>50] <- NA
  ## 
  i <- grep("^Sol|^G",names(X)); X[ ,i][X[ ,i]<0] <- 0
  ##
  invisible(X)
}

## Load the data
X <- readIDEEdata("5min_prbs1_qv", "14-02-01")
plotmulti(X, c("^T|^yT","^P","^Sol|^G","^Ach","^WS"))
## Keep only some of the signals
X <- X[,c("t","Ph","Ti","Te","Gv")]
names(X)
X <- X[per("14-02-11",X$t,"14-02-18"),]
## Write to a file
save(X, file="dataIDEE/prbs1.rda")

## @knitr greyboxForIDEE_init
## Load the helping functions
source("sourceFunctions.R")

## @knitr greyboxForIDEE_plotseries1
## Load the data (the data.frame X is stored in the file)
load("dataIDEE/prbs1.rda")

## Plot the raw 5 minutes series
setpar(mfrow=c(3,1))
plot(X$t, X$Ph, type="l", xlab="", ylab="$\\Phi_{\\mathrm{h}}$ (W)")
plot(X$t, X$Ti, type="l", ylim=range(X$Ti,X$Te), xlab="", ylab="Temperature ($^\\circ$C)")
lines(X$t, X$Te, col=2)
legend("right", c("Internal $T_{\\text{i}}$","External $T_{\\text{e}}$"), col=1:2, lty=1)
plot(X$t, X$Gv, type="l", xlab="", ylab="Gv (W/m$^2$)")
axis.POSIXct(1, X$t, xaxt="s")

## @knitr greyboxForIDEE_plotseries2
## Plot a single day
setpar(mfrow=c(3,1))
tmp <- X[per("14-02-12 05:00",X$t,"14-02-12 19:00"), ]
plot(tmp$t, tmp$Ph, type="b", xlab="", ylab="$\\Phi_{\\mathrm{h}}$ (W)", main="2014-02-02", cex=0.8)
plot(tmp$t, tmp$Ti, type="b", ylim=range(tmp$Ti,tmp$Te), xlab="", ylab="Temperature ($^\\circ$C)", cex=0.8)
lines(tmp$t, tmp$Te, type="b", col=2, cex=0.8)
legend("right", c("Internal $T_{\\mathrm{i}}$","External $T_{\\mathrm{e}}$"), col=1:2, lty=1)
plot(tmp$t, tmp$Gv, type="b", xlab="", ylab="Gv (W/m$^2$)", cex=0.8)
axis.POSIXct(1, tmp$t, xaxt="s")

## @knitr greyboxForIDEE_fit5minSingleState
## Fit the simplest single-state Grey-box model to the 5 minutes series
load("dataIDEE/prbs1.rda")
## Use the ctsmr package, see http://ctsm.info for installation
require(ctsmr)
## A list for some global parameters
pg <- list()
## Number of threads to be used in optimization
pg$threads <- 4
## Use a first order resampling scheme between the data points
pg$firstorder <- TRUE
## Convert the time t to seconds
X$timedate <- X$t
X$t <- as.numeric(X$t-X$t[1], units="secs")
## Fit the model using the global radiation as incomming solar radiation to the building
X$Isol <- X$Gv
## In the model definition the output is set to yTi
X$yTi <- X$Ti
## Define and estimate the single-state model, see the function defined in "functions/sdeTi.R"
names(X)
fit <- sdeTi(X)

## @knitr greyboxForIDEE_valSeries5minSingleState
## Analyse the fit using the helping function, see "functions/analyzeGBFit.R"
## Plot the series and the one-step ahead residuals, and the residuals ACF and CPGRAM
analyzeGBFit(fit, plotSeries=TRUE)

## @knitr greyboxForIDEE_valACF5minSingleState
analyzeGBFit(fit, plotACF=TRUE)

## @knitr greyboxForIDEE_fit5minTwoState
## Now fit the two-state model
fit <- sdeTiTw(X)

## @knitr greyboxForIDEE_valSeries5minTwoState
## Plot the series and the one-step ahead residuals
analyzeGBFit(fit, plotSeries=TRUE)

## @knitr greyboxForIDEE_valACF5minTwoState
## Plot the residuals ACF and CPGRAM
analyzeGBFit(fit, plotACF=TRUE)

## @knitr greyboxForIDEE_fit10minTwoState
## Resample to 10 minutes average values
Xres <- resampleDF(X, Ts=10*60, startTime=X$t[1])
## Fit the two-state model to the resampled data
fit <- sdeTiTw(Xres)

## @knitr greyboxForIDEE_valSeries10minTwoState
## Plot the series and the one-step ahead residuals
analyzeGBFit(fit, plotSeries=TRUE)

## @knitr greyboxForIDEE_valACF10minTwoState
## Plot the residuals ACF and CPGRAM
analyzeGBFit(fit, plotACF=TRUE)

## @knitr greyboxForIDEE_valSummary10minTwoState
analyzeGBFit(fit, printSummary=TRUE)

